@@ -1,5 +1,5 @@
 //
-// $Id: dnstracer.c,v 1.31 2002/02/24 21:59:23 mavetju Exp $
+// $Id: dnstracer.c,v 1.35 2002/06/12 04:16:04 mavetju Exp $
 //
 
 //
@@ -484,11 +484,11 @@ char *get_resource(int type,struct dnssession *session,char *buffer,int dots) {
 	return retval;
 
     case ns_t_cname:
-	sprintf(retval,printablename(getname(session,&buffer),dots));
+	strcpy(retval,printablename(getname(session,&buffer),dots));
 	return retval;
 
     case ns_t_txt:
-	sprintf(retval,printablename(getname(session,&buffer),dots));
+	strcpy(retval,printablename(getname(session,&buffer),dots));
 	return retval;
 
     case ns_t_mx:
@@ -502,11 +502,15 @@ char *get_resource(int type,struct dnssession *session,char *buffer,int dots) {
     }
 
     case ns_t_hinfo:
-	sprintf(retval,printablename(getname(session,&buffer),dots));
+	strcpy(retval,printablename(getname(session,&buffer),dots));
 	return retval;
 
     case ns_t_ns:
-	sprintf(retval,printablename(getname(session,&buffer),dots));
+	strcpy(retval,printablename(getname(session,&buffer),dots));
+	return retval;
+
+    case ns_t_ptr:
+	strcpy(retval,printablename(getname(session,&buffer),dots));
 	return retval;
 
     default: return "unknown";
@@ -1093,8 +1097,10 @@ int create_session(char *host,
 	// of what we have, don't go there. This might happen if we are
 	// looking through cnames from different domains.
 	//
-	if (strcasecmp(rrauth->domainname_string,
-		host+strlen(host)-strlen(rrauth->domainname_string))!=0) {
+	if (strcmp(rrauth->domainname_string,".") &&
+		strcasecmp(rrauth->domainname_string,
+		    host+strlen(host)-strlen(rrauth->domainname_string))!=0) {
+	    printf("blaat '%s' '%s'\n",host,rrauth->domainname_string);
 	    rrauth=rrauth->next;
 	    continue;
 	}
@@ -1224,7 +1230,7 @@ int create_session(char *host,
 
 void usage(void) {
     fprintf(stderr,
-	"DNSTRACER version 1.4 - (c) Edwin Groothuis - http://www.mavetju.org\n"
+	"DNSTRACER version 1.5 - (c) Edwin Groothuis - http://www.mavetju.org\n"
 	"Usage: dnstracer [options] [host]\n"
 	"\t-c: disable local caching, default enabled\n"
 	"\t-o: enable overview of received answers, default disabled\n"
@@ -1256,6 +1262,7 @@ int main(int argc,char **argv) {
     char *	server_name="127.0.0.1";
     char *	server_ip="127.0.0.1";
     char	ipaddress[NS_MAXDNAME];
+    char	argv0[NS_MAXDNAME];
     int		server_root=0;
 
 #ifndef WIN32
@@ -1295,6 +1302,7 @@ int main(int argc,char **argv) {
 		compare("a",	ns_t_a		);
 		compare("aaaa",	ns_t_aaaa	);
 		compare("a6",	ns_t_aaaa	);
+		compare("ptr",	ns_t_ptr	);
 		compare("cname",ns_t_cname	);
 		compare("hinfo",ns_t_hinfo	);
 		compare("mx",	ns_t_mx		);
@@ -1346,8 +1354,12 @@ int main(int argc,char **argv) {
 
     if (argv[0]==NULL) usage();
 
+    // check for a trailing dot
+    strcpy(argv0,argv[0]);
+    if (argv0[strlen(argv[0])-1]=='.') argv0[strlen(argv[0])-1]=0;
+
     printf("Tracing to %s via %s, timeout %d seconds\n",
-	argv[0],server_name,global_timeout);
+	argv0,server_name,global_timeout);
 
     srandom(time(NULL));
 
@@ -1364,7 +1376,7 @@ int main(int argc,char **argv) {
 	    (unsigned char)h->h_addr_list[0][3]);
     }
 
-    create_session(argv[0],ipaddress,server_name,server_root==0?NULL:".",0,"",1);
+    create_session(argv0,ipaddress,server_name,server_root==0?NULL:".",0,"",1);
 
     printf("\n");
 
